#include <MDR32FxQI_port.h>
#include <MDR32FxQI_rst_clk.h>
#include <MDR32FxQI_ssp.h>
#include <MDR32FxQI_eeprom.h>
#include <MDR32FxQI_bkp.h>

void Delay(int waitTicks);

#define VD3 PORT_Pin_0
#define VD4 PORT_Pin_1

#define LED_ALL VD3 | VD4

#define MASTER
#define PULLDOWN_CLK_SLAVE

uint32_t RXPacket;
uint32_t TXCount;
uint32_t RXSuccess;
uint32_t key;

int main()
{
    PORT_InitTypeDef GPIOInitStruct;
    SSP_InitTypeDef SSP;

    RST_CLK_DeInit();

    RST_CLK_HSEconfig (RST_CLK_HSE_ON); // HSE Enable
    while (RST_CLK_HSEstatus() != SUCCESS); // Wait HSE enable

    RST_CLK_CPU_PLLcmd (ENABLE); // PLL Enable

    RST_CLK_CPU_PLLconfig (RST_CLK_CPU_PLLsrcHSEdiv1, RST_CLK_CPU_PLLmul10); // Set 80 Mhz for out PLL
    while (RST_CLK_CPU_PLLstatus() != SUCCESS);

    RST_CLK_CPU_PLLuse (ENABLE);

    RST_CLK_CPUclkPrescaler (RST_CLK_CPUclkDIV1);

    RST_CLK_PCLKcmd (RST_CLK_PCLK_EEPROM, ENABLE);
    EEPROM_SetLatency(EEPROM_Latency_3); // Set delay EEPROM for correct work by frequency 80 MHz
    RST_CLK_PCLKcmd (RST_CLK_PCLK_EEPROM, DISABLE);

    RST_CLK_CPUclkSelection (RST_CLK_CPUclkCPU_C3); // Set 80 MHz for work

    RST_CLK_PCLKcmd (RST_CLK_PCLK_PORTC | RST_CLK_PCLK_PORTF | RST_CLK_PCLK_SSP1, ENABLE);
    MDR_RST_CLK -> SSP_CLOCK |= (1 << 24); // Enable clock SSP1
    PORT_StructInit(&GPIOInitStruct);

    GPIOInitStruct.PORT_Pin        = LED_ALL; // LED init
    GPIOInitStruct.PORT_OE         = PORT_OE_OUT;
    GPIOInitStruct.PORT_SPEED      = PORT_SPEED_SLOW;
    GPIOInitStruct.PORT_MODE       = PORT_MODE_DIGITAL;
    PORT_Init(MDR_PORTC, &GPIOInitStruct);

    GPIOInitStruct.PORT_Pin        = PORT_Pin_2; // Button SELECT init
    GPIOInitStruct.PORT_OE         = PORT_OE_IN;
    PORT_Init(MDR_PORTC, &GPIOInitStruct);

    GPIOInitStruct.PORT_Pin        = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2; // SPI init. PF0 - TXD, PF1 - CLK, PF2 - FSS
//  PORT_OE controlled by SSP in alternative function, no need to init
    GPIOInitStruct.PORT_SPEED      = PORT_SPEED_MAXFAST;
    GPIOInitStruct.PORT_FUNC       = PORT_FUNC_ALTER;
    GPIOInitStruct.PORT_MODE       = PORT_MODE_DIGITAL;
    PORT_Init(MDR_PORTF, &GPIOInitStruct);

#if (defined PULLDOWN_CLK_SLAVE & defined SLAVE) // Settings PF1 for correct work without FSS by mode for SPH = 1
    GPIOInitStruct.PORT_Pin        = PORT_Pin_1;
    GPIOInitStruct.PORT_PULL_DOWN  = PORT_PULL_DOWN_ON;
    PORT_Init(MDR_PORTF, &GPIOInitStruct);
    GPIOInitStruct.PORT_PULL_DOWN  = PORT_PULL_DOWN_OFF;
#endif
    
    GPIOInitStruct.PORT_Pin        = PORT_Pin_3; // SPI init. PF3 - RXD
//  PORT_OE controlled by SSP in alternative function, no need to init
    PORT_Init(MDR_PORTF, &GPIOInitStruct);
    
    SSP_DeInit(MDR_SSP1);
    SSP_StructInit(&SSP);
    
#if (defined MASTER)
    SSP.SSP_Mode = SSP_ModeMaster;
#elif (defined SLAVE)
    SSP.SSP_Mode = SSP_ModeSlave;
#endif

    SSP.SSP_SCR = 0x0;
    SSP.SSP_CPSDVSR = 0xC; // Divider = 12. 80 MHz/12 = 6.6 MHz
    SSP.SSP_WordLength = SSP_WordLength8b;
    SSP.SSP_SPH = SSP_SPH_1Edge; // SPH = 0
    SSP.SSP_SPO = SSP_SPO_Low; // SPO = 0
    SSP.SSP_FRF = SSP_FRF_SPI_Motorola;

    SSP_Init(MDR_SSP1, &SSP);

    SSP_Cmd(MDR_SSP1, ENABLE);

    while (1)
    {
#if (defined MASTER)
        while(PORT_ReadInputDataBit(MDR_PORTC, PORT_Pin_2) == RESET)
        {
            for(uint32_t i = 0; i <= 0xFF; i++)
            {
                while((MDR_SSP1->SR & SSP_FLAG_TFE) == 0 ) {}
                SSP_SendData(MDR_SSP1, i);

                while((MDR_SSP1->SR & SSP_FLAG_RNE) == 0 ) {}
                RXPacket = SSP_ReceiveData(MDR_SSP1);

                if(RXPacket == i)
                {
                    RXSuccess++;
                }
                TXCount++;
            }
            key = 1;
        }

        if((RXSuccess == TXCount) & (key == 1))
        {
            PORT_SetBits(MDR_PORTC, LED_ALL);
            RXSuccess = 0;
            TXCount = 0;
            key = 0;
        }
        else if ((RXSuccess != TXCount) & (key == 1))
        {
            PORT_ResetBits(MDR_PORTC, LED_ALL);
            PORT_SetBits(MDR_PORTC, VD3);
            RXSuccess = 0;
            TXCount = 0;
            key = 0;
        }

#elif (defined SLAVE)
        while((MDR_SSP1->SR & SSP_FLAG_RNE) == 0 ) {}
        RXPacket = SSP_ReceiveData(MDR_SSP1);

        while((MDR_SSP1->SR & SSP_FLAG_TFE) == 0 ) {}
        SSP_SendData(MDR_SSP1, RXPacket+1);
#endif
    }
}


void Delay(int waitTicks)
{
    int i;
    for (i = 0; i < waitTicks; i++)
    {
        __NOP();
    }
}

